﻿using System;
using System.IO;
using System.Net;
using System.Text;
using System.Windows;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Notification;

namespace RawNotification
{
    public partial class MainPage : PhoneApplicationPage
    {
        //푸쉬 채널생성
        HttpNotificationChannel pushChannel;
        string channelUri;

        //푸쉬 채널이름 설정
        string channelName = "Kaki104RawNotificationChannel";

        // Constructor
        public MainPage()
        {
            InitializeComponent();

            this.Loaded += new RoutedEventHandler(MainPage_Loaded);
        }

        void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            //채널이름 찾아보기
            pushChannel = HttpNotificationChannel.Find(channelName);
            if (pushChannel == null)
            {
                //생성된 채널이 없으면 생성
                pushChannel = new HttpNotificationChannel(channelName);
                //채널정보 업데이트시 이벤트 연결
                pushChannel.ChannelUriUpdated += new EventHandler<NotificationChannelUriEventArgs>(pushChannel_ChannelUriUpdated);
                //채널 오류 발생 이벤트 연결
                pushChannel.ErrorOccurred += new EventHandler<NotificationChannelErrorEventArgs>(pushChannel_ErrorOccurred);
                //채널 열기
                pushChannel.Open();
            }
            else
            {
                //채널 업데이트 이벤트 연결
                pushChannel.ChannelUriUpdated += new EventHandler<NotificationChannelUriEventArgs>(pushChannel_ChannelUriUpdated);
                //채널 오류 발생 이벤트 연결
                pushChannel.ErrorOccurred += new EventHandler<NotificationChannelErrorEventArgs>(pushChannel_ErrorOccurred);
                //채널 Uri정보 조회
                System.Diagnostics.Debug.WriteLine("[ChannelUri] " + pushChannel.ChannelUri.ToString());

                //채널 정보를 미리 저장
                channelUri = pushChannel.ChannelUri.ToString();
            }
        }

        void pushChannel_ChannelUriUpdated(object sender, NotificationChannelUriEventArgs e)
        {
            //sender를 푸시체널로 형변환하고
            HttpNotificationChannel pushChannel = sender as HttpNotificationChannel;
            //푸시체널을 어플리케이션타일과 바인딩
            //pushChannel.BindToShellTile();

            //이넘이 raw notification 인듯
            pushChannel.HttpNotificationReceived += new EventHandler<HttpNotificationEventArgs>(pushChannel_HttpNotificationReceived);

            //변경된 채널 Uri정보 디버그 창에 출력
            System.Diagnostics.Debug.WriteLine("[ChannelUri] " + e.ChannelUri.ToString());

            //채널 정보를 미리 저장
            channelUri = pushChannel.ChannelUri.ToString();
        }

        //Raw Notification 처리
        void pushChannel_HttpNotificationReceived(object sender, HttpNotificationEventArgs e)
        {
            if (e.Notification.Body != null && e.Notification.Headers != null) 
            {
                StreamReader reader = new StreamReader(e.Notification.Body);
                Dispatcher.BeginInvoke(() => 
                {
                    Receive.Text = reader.ReadToEnd();
                });
            }
        }

        //Error
        void pushChannel_ErrorOccurred(object sender, NotificationChannelErrorEventArgs e)
        {
            //발생된 에러 메시지 조회
            MessageBox.Show(String.Format("푸쉬 노티피케이션 {0} 에러 발생.  {1} ({2}) {3}",
                                e.ErrorType, e.Message, e.ErrorCode, e.ErrorAdditionalData)
                                );
        }

        //Send Button 
        private void Button_Click(object sender, RoutedEventArgs e)
        {
            SendRawNotification();
        }

        //Self Raw Notification Send
        private void SendRawNotification()
        {
            uri.Text = channelUri;
            Uri sendUri = new Uri(uri.Text, UriKind.Absolute);

            //Request 생성
            HttpWebRequest request = (HttpWebRequest)WebRequest.Create(sendUri);
            //POST로 지정
            request.Method = "POST";
            //뒤에 utf-8을 넣어 주어야 한글도 깨지지 않고 넘어감
            request.ContentType = "text/xml;charset=utf-8";
            //Raw Notification
            request.Headers["X-NotificationClass"] = "3";
            
            request.BeginGetRequestStream(new AsyncCallback(channelRequest), request);
        }

        //보내는 곳과 연결 하고
        private void channelRequest(IAsyncResult asyncResult)
        {
            Dispatcher.BeginInvoke(() => 
            {
                HttpWebRequest request = (HttpWebRequest)asyncResult.AsyncState;
                //몸통 만들고
                Stream body = request.EndGetRequestStream(asyncResult);
                //전송할 데이터 만들고
                byte[] contents = Encoding.UTF8.GetBytes(tbMessage.Text);
                //전송
                body.Write(contents, 0, contents.Length);
                body.Flush();
                body.Close();

                request.BeginGetResponse(new AsyncCallback(channelResponse), request);
            });
        }

        private void channelResponse(IAsyncResult asyncResult)
        {
            string notificationStatus = "";
            string notificationChannelStatus = "";
            string deviceConnectionStatus = "";

            var request = (HttpWebRequest)asyncResult.AsyncState;
            var response = request.EndGetResponse(asyncResult);
            //전송 결과 뽑아내고
            notificationStatus = response.Headers["X-NotificationStatus"];
            notificationChannelStatus = response.Headers["X-SubscriptionStatus"];
            deviceConnectionStatus = response.Headers["X-DeviceConnectionStatus"];
            //표시해준다
            Dispatcher.BeginInvoke(() => 
            {
                tbResponse.Text = notificationStatus
                                    + " " + notificationChannelStatus
                                    + " " + deviceConnectionStatus;
            });
        }

    }
}